# Copyright (c) 2025 Bytedance Ltd. and/or its affiliates
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from arkitect.core.component.tts.base import AsyncBaseTTSClient, TTSResponseChunk
from arkitect.core.component.tts.bot_util import create_bot_audio_responses
from arkitect.core.component.tts.model import AudioParams, ConnectionParams, TextRequest
from arkitect.core.component.tts.tts_client import AsyncTTSClient

__all__ = [
    "AsyncBaseTTSClient",
    "TTSResponseChunk",
    "AsyncTTSClient",
    "ConnectionParams",
    "AudioParams",
    "TextRequest",
    "create_bot_audio_responses",
]
